/*
  Copyright (C) 2007 Kochetkov Nikita <swappp at gmail dot com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <QMetaProperty>
#include <QComboBox>
#include "properties.h"

QVariant PropertiesModel::data(const QModelIndex &index, int role) const
{
	QObject* o;
	if (!index.isValid())
		return QVariant();
	if (index.internalPointer())
		o = static_cast<QObject*>(index.internalPointer());
	else
		o = obj;
	if (role == Qt::DisplayRole || role == Qt::EditRole)
	{
		if (index.column() == 0)
			return QVariant(tr(o->metaObject()->property(index.row()).name()));
		else if (o->metaObject()->property(index.row()).isWritable())
		{
			if (o->metaObject()->property(index.row()).isEnumType())
				return QVariant(o->metaObject()->property(index.row()).enumerator().key(o->metaObject()->property(index.row()).read(o).toInt()));
			return o->metaObject()->property(index.row()).read(o);
		}
	}
	return QVariant();
}
int PropertiesModel::columnCount(const QModelIndex &parent) const
{
	return 2;
}
int PropertiesModel::rowCount(const QModelIndex &parent) const
{
	if (parent.column() > 0)
		return 0;
	QObject* o;
	if (!parent.isValid())
		o = obj;
	else
	{
		if (parent.internalPointer())
		{
			o = static_cast<QObject*>(parent.internalPointer());
			o = o->metaObject()->property(parent.row()).read(o).value<QObject*>();
			if (!o)
				return 0;
		}
		else
			return 0;
	}
	return o->metaObject()->propertyCount();
}
QModelIndex PropertiesModel::index(int row, int column, const QModelIndex &parent) const
{
	QObject* o;
	if (parent.isValid() && parent.internalPointer())
	{
		o = static_cast<QObject*>(parent.internalPointer());
		o = o->metaObject()->property(parent.row()).read(o).value<QObject*>();
		if (!o)
			return QModelIndex();
	}
	else
		o = obj;
	if (row >= 0 && o->metaObject()->property(row).isUser())
		return createIndex(row, column, o);
	return QModelIndex();
}
QModelIndex PropertiesModel::parent(const QModelIndex &index) const
{
	if (!index.isValid())
		return QModelIndex();
	if (QObject* o = static_cast<QObject*>(index.internalPointer()))
	{
		if (o != obj)
			if (QObject* p = o->parent())
				for (int i = 0; i < p->metaObject()->propertyCount(); i++)
					if (p->metaObject()->property(i).isUser() && p->metaObject()->property(i).read(p).value<QObject*>() == o)
						return createIndex(i, 0, p);
	}
	return QModelIndex();
}
Qt::ItemFlags PropertiesModel::flags(const QModelIndex &index) const
{
	if (!index.isValid())
		return Qt::ItemIsEnabled;
	if (index.column() == 1 && static_cast<QObject*>(index.internalPointer())->metaObject()->property(index.row()).isWritable())
		return QAbstractItemModel::flags(index) | Qt::ItemIsEditable;
	return QAbstractItemModel::flags(index);
}
bool PropertiesModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
	if (index.isValid() && role == Qt::EditRole)
	{
		QObject *o = static_cast<QObject*>(index.internalPointer());
		o->metaObject()->property(index.row()).write(o,value);
		emit dataChanged(index, index);
		emit layoutChanged();
		return true;
	}
	return false;
}
QVariant PropertiesModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (orientation == Qt::Horizontal && role == Qt::DisplayRole)
    {
    	if (section == 0)
    		return tr("Property");
    	else if (section == 1)
    		return tr("Value");
    }
    return QVariant();
}

QWidget* PropertiesDelegate::createEditor(QWidget * parent, const QStyleOptionViewItem & option, const QModelIndex & index) const
{
	if (index.isValid() && index.internalPointer())
	{
		QObject* o = static_cast<QObject*>(index.internalPointer());
		if (o->metaObject()->property(index.row()).isEnumType())
		{
			QComboBox* combo = new QComboBox(parent);
			for (int i = 0; i < o->metaObject()->property(index.row()).enumerator().keyCount(); i++)
				combo->insertItem(i, tr(o->metaObject()->property(index.row()).enumerator().key(i)));
			combo->setCurrentIndex(o->metaObject()->property(index.row()).read(o).toInt());
			return combo;
		}
	}
	return QItemDelegate::createEditor(parent, option, index);
}
void PropertiesDelegate::setEditorData(QWidget * editor, const QModelIndex & index) const
{
	if (index.isValid() && index.internalPointer())
	{
		QObject* o = static_cast<QObject*>(index.internalPointer());
		if (o->metaObject()->property(index.row()).isEnumType())
			if (QComboBox* combo = qobject_cast<QComboBox*>(editor))
				o->metaObject()->property(index.row()).write(o, QVariant(combo->currentIndex()));
	}
	QItemDelegate::setEditorData(editor, index);
}
